import * as vscode from 'vscode';
import axios from 'axios';
import { exec } from 'child_process';

// Funktion zum Erstellen eines Pull Requests
async function createGiteaPullRequest() {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage('Gitea URL oder Token ist nicht konfiguriert.');
        return;
    }

    // Aktuellen Git-Ordner abrufen
    const currentWorkspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri?.fsPath;
    if (!currentWorkspaceFolder) {
        vscode.window.showErrorMessage('Kein gültiges Git-Repository im aktuellen Workspace gefunden.');
        return;
    }

    // Git-Repository-Informationen abrufen
    exec(`git config --get remote.origin.url`, { cwd: currentWorkspaceFolder }, async (error, stdout, stderr) => {
        if (error || !stdout) {
            vscode.window.showErrorMessage('Konnte die Git-Remote-URL nicht abrufen.');
            return;
        }

        let repoUrl = stdout.trim();

        // URL umwandeln, wenn sie im SSH-Format vorliegt (git@gitea.com:owner/repo.git)
        if (repoUrl.startsWith('git@')) {
            const parts = repoUrl.split(':');
            const domain = parts[0].replace('git@', '');
            const path = parts[1].replace('.git', '');
            repoUrl = `https://${domain}/${path}`;
        } else {
            repoUrl = repoUrl.replace('.git', '');
        }

        // Extract repository owner and name from the URL
        const [owner, repo] = repoUrl.split('/').slice(-2);

        try {
            // Den letzten Commit und den Branch abrufen
            const { title, body } = await getLastCommit(currentWorkspaceFolder);
            const branch = await getCurrentBranch(currentWorkspaceFolder);

            if (!branch) {
                vscode.window.showErrorMessage('Branch konnte nicht ermittelt werden.');
                return;
            }

            // Base-Branch über die Gitea API ermitteln
            const baseBranch = await getDefaultBranch(instanceUrl, owner, repo, token);

            // API-Request-Daten vorbereiten
            const prData = {
                title: title, // Der letzte Commit als Titel
                body: body || '', // Commit-Kommentare als Body
                head: branch, // Der aktuelle Branch als "head"
                base: baseBranch // Der ermittelte base-Branch
            };

            const config = {
                method: 'post',
                maxBodyLength: Infinity,
                url: `${instanceUrl}/api/v1/repos/${owner}/${repo}/pulls`,
                headers: {
                    'Content-Type': 'application/json',
                    'Authorization': `token ${token}`
                },
                data: JSON.stringify(prData)
            };

            // API-Request zum Erstellen des Pull Requests
            const response = await axios.request(config);

            const prUrl = response.data.html_url;

            // Öffne die URL des erstellten PRs im Browser
            vscode.env.openExternal(vscode.Uri.parse(prUrl));
            vscode.window.showInformationMessage('Pull-Request erfolgreich erstellt und im Browser geöffnet.');
        } catch (err: any) {
            vscode.window.showErrorMessage(`Fehler beim Erstellen des Pull Requests: ${err.message}`);
            console.error('Fehler beim Erstellen des PRs:', err);
        }
    });
}

// Hilfsfunktion, um den letzten Commit zu ermitteln
async function getLastCommit(folderPath: string): Promise<{ title: string, body: string }> {
    return new Promise((resolve, reject) => {
        exec(`git log -1 --pretty=format:"%s%n%b"`, { cwd: folderPath }, (error, stdout) => {
            if (error) {
                reject('Fehler beim Abrufen des letzten Commits.');
            } else {
                const output = stdout.split('\n');
                const title = output[0]; // Commit-Message als Titel
                const body = output.slice(1).join('\n'); // Commit-Kommentar als Body
                resolve({ title: title.trim(), body: body.trim() });
            }
        });
    });
}

// Hilfsfunktion, um den aktuellen Branch zu ermitteln
async function getCurrentBranch(folderPath: string): Promise<string> {
    return new Promise((resolve, reject) => {
        exec(`git rev-parse --abbrev-ref HEAD`, { cwd: folderPath }, (error, stdout) => {
            if (error) {
                reject('Fehler beim Abrufen des Branches.');
            } else {
                resolve(stdout.trim());
            }
        });
    });
}

// Hilfsfunktion, um den base-Branch über die Gitea API zu ermitteln
async function getDefaultBranch(instanceUrl: string, owner: string, repo: string, token: string): Promise<string> {
    try {
        const response = await axios.get(`${instanceUrl}/api/v1/repos/${owner}/${repo}`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200 && response.data.default_branch) {
            return response.data.default_branch;
        } else {
            vscode.window.showErrorMessage('Konnte den base-Branch nicht ermitteln. Standardmäßig "main" verwendet.');
            return 'main';
        }
    } catch (error) {
        vscode.window.showErrorMessage('Fehler beim Abrufen des base-Branches. Standardmäßig "main" verwendet.');
        return 'main';
    }
}

// Funktion zum Hinzufügen des Statusbar-Icons
function addStatusBarIcon() {
    const statusBar = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left);
    statusBar.text = `$(git-pull-request) Create Gitea PR`;
    statusBar.tooltip = 'Create a Pull Request in Gitea';
    statusBar.command = 'gitea.createPullRequest';
    statusBar.show(); // Sofortiges Anzeigen des Icons in der Statusleiste
    return statusBar;
}

// Funktion zur Authentifizierung bei Gitea mit dem Personal Access Token (PAT)
async function authenticateGitea() {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage('Gitea URL oder Token ist nicht konfiguriert.');
        return;
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/user`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200) {
            vscode.window.showInformationMessage(`Authentifizierung erfolgreich: ${response.data.username}`);
        }
    } catch (error) {
        vscode.window.showErrorMessage('Authentifizierung fehlgeschlagen.');
        console.error(error);
    }
}

// Funktion zur Konfiguration der Gitea-Instanz und des Tokens über die Command Palette
async function configureGitea() {
    const instanceUrl = await vscode.window.showInputBox({
        prompt: "Gitea Instanz-URL eingeben",
        placeHolder: "https://your-gitea-instance.com"
    });

    const token = await vscode.window.showInputBox({
        prompt: "Gitea Personal Access Token eingeben",
        placeHolder: "token",
        password: true
    });

    if (instanceUrl && token) {
        await vscode.workspace.getConfiguration().update('gitea.instanceUrl', instanceUrl, vscode.ConfigurationTarget.Global);
        await vscode.workspace.getConfiguration().update('gitea.personalAccessToken', token, vscode.ConfigurationTarget.Global);
        vscode.window.showInformationMessage('Gitea Konfiguration aktualisiert.');
    }
}

// Aktivierungsfunktion des Plugins
export function activate(context: vscode.ExtensionContext) {
    // Registriert den Befehl zur Authentifizierung
    let authCommand = vscode.commands.registerCommand('gitea.authenticate', authenticateGitea);
    context.subscriptions.push(authCommand);

    // Registriert den Befehl zur Konfiguration
    let configCommand = vscode.commands.registerCommand('gitea.configure', configureGitea);
    context.subscriptions.push(configCommand);

    // Registriere den Befehl zum Erstellen eines Pull Requests
    let pullRequestCommand = vscode.commands.registerCommand('gitea.createPullRequest', createGiteaPullRequest);
    context.subscriptions.push(pullRequestCommand);

    // Statusbar-Icon sofort anzeigen
    const statusBar = addStatusBarIcon(); // Icon wird sofort beim Aktivieren angezeigt
    context.subscriptions.push(statusBar);
}

// Deaktivierungsfunktion des Plugins
export function deactivate() {}
